#include "EditMainWin.hpp"
#include "EditRC.h"
#include <ifiledlg.hpp>
#include <icconst.h>


Editor::Editor(){
  frame=new IFrameWindow(1,  IFrameWindow::titleBar|
                             IFrameWindow::sizingBorder|
                             IFrameWindow::minimizeButton|
                             IFrameWindow::maximizeButton|
                             IFrameWindow::windowList|
                             IFrameWindow::menuBar|
                             IFrameWindow::systemMenu);

  handleEventsFor(frame);

  editwin=new IMultiLineEdit(10,frame,frame);

  frame->setClient(editwin);
  toolbar=new EditToolBarSet(frame);
  frame->show();
}

Editor::~Editor(){
  delete editwin;
  delete toolbar;
  delete frame;
}

Boolean Editor::command(ICommandEvent &event){

  switch(event.commandId()){
     case MenNew:
        FileName="";
        editwin->removeAll();
        break;
     case MenExit:
        IApplication::current().exit();
        break;
     case MenSave:
        if(FileName.length()){
           editwin->exportToFile(FileName);
           break;
        }
     case MenSaveAs:{
        IFileDialog::Settings settings;
        settings.setSaveAsDialog();
        IFileDialog  fd(IWindow::desktopWindow(),frame,settings);

        if(fd.pressedOK()){
           FileName=fd.fileName();
           editwin->exportToFile(FileName);
        }
        break;
     }
     case MenLoad:{
        IFileDialog::Settings settings;
        settings.setFileName("*.txt");
        IFileDialog  fd(IWindow::desktopWindow(),frame,settings);

        if(fd.pressedOK()){
           FileName=fd.fileName();
           editwin->importFromFile(FileName);
        }
        break;
     }
     default:
        return false;
  }

return true;
}

/**********************************************************************/

EditToolBarSet::EditToolBarSet(IFrameWindow *wnd):
              IToolBar(
                 1111,
                 wnd,
                 aboveClient,
                 false,
                 defaultStyle()
              ),
              area(
                 wnd,
                 1112
              ),
              bLoad(
                 MenLoad,
                 this,
                 this,
                 IRectangle(),
                 IToolBarButton::defaultStyle()
              ),
              bSave(
                 MenSave,
                 this,
                 this,
                 IRectangle(),
                 IToolBarButton::defaultStyle()
              ),
              flytext(
                 1113,
                 this
              ),
              flyHand(
                 &flytext,
                 &area
              )
{
  bLoad.setBitmap(IC_ID_OPEN);
  bLoad.setText("Open");

  bSave.setBitmap(IC_ID_SAVE);
  bSave.setText("Save");

  addAsLast(&bLoad);
  addAsLast(&bSave);

  flyHand.setFlyTextStringTableOffset( FlyBase );
  flyHand.setLongStringTableOffset( LongBase );
  flyHand.handleEventsFor(this);
}

EditToolBarSet::~EditToolBarSet(){
}
