/* Decodes CP info & CP data sectors on a HPFS volume. Requires SECTOR.DLL */

ARG drive .       /* First parm should always be drive */
IF drive = '' | drive = "?" | drive = "HELP",
              | drive = "A:" | drive = "B:" THEN CALL Help

CALL RxFuncAdd 'ReadSect','Sector','ReadSect'  /* In SECTOR.DLL */
secString = ReadSect(drive,17)     /* SpareBlock is LSN 17 */
'@cls'
SAY
SAY "Inspecting drive" drive
SAY
/* Offset 33 in Spareblock contains dword of CP info LSN */
cpInfoSec = C2D(Reverse(Substr(secString,33,2)))
secString = ReadSect(drive,cpInfoSec)   /* Load CP info sec */
numOfCodePages = C2D(Reverse(Substr(secString,5,2)))
prevDataSec = ''

SAY "CODE PAGE INFORMATION (sector" cpInfoSec"):"
SAY "Signature Dword: 0x"FourChars2Hex(1)
SAY " CP#  Ctry Code     Code Page      CP Data Sec     Offset"

DO x = 0 TO numOfCodePages-1
   hexCountry = TwoChars2Hex((16*x)+17)
   decCountry = Right('00'X2D(hexCountry),3)
   cp = TwoChars2Hex((16*x)+19)
   country.x = X2D(cp)
   hexSec = FourChars2Hex((16*x)+25)
   decSec = X2D(hexSec)
   cpDataSec = decSec
   /* Since up to 3 CP tables can fit in 1 CP data sec,
      only read in a new data sec when the need arises. */
   IF cpDataSec \= prevDataSec THEN
      DO
      dataSecString = ReadSect(drive,cpDataSec)
      prevDataSec = cpDataSec
      END

   offset = C2D(Reverse(Substr(dataSecString,(2*x)+21,2)))
   start = offset + 1
   SAY " " x " 0x"hexCountry "("decCountry")  0x"cp "("X2D(cp)")  0x"hexSec "("decSec")  0x"D2X(offset) "("offset")"

   /* Store table contents of each CP in an array */
   DO y = 128 TO 255
      char = Substr(dataSecString,start+6+y-128,1)
      IF C2D(char) \= y THEN
         array.x.y = Format(C2D(char),4) "("char")"
      ELSE
         array.x.y = " same   "
   END y
END x

/* Work out title line based on number of CPs */
titleLine = "  ASCII  "

DO x = 0 TO numOfCodePages-1
   titleLine = titleLine "   CP" country.x
END x

SAY
SAY titleLine

/* Display each table entry based on number of CPs */
DO y = 128 TO 255
   dispLine = ''

   DO x = 0 TO numOfCodePages-1
      dispLine = dispLine"  "array.x.y
   END x

   SAY "" y "("D2C(y)"):" dispLine
END y

EXIT  /****************EXECUTION ENDS HERE****************/


FourChars2Hex:
ARG offset
RETURN C2X(Reverse(Substr(secString,offset,4)))


TwoChars2Hex:
ARG offset
RETURN C2X(Reverse(Substr(secString,offset,2)))


Help:  
SAY "Purpose:"
SAY "  CP    decodes the CodePage Directory sector &"
SAY "                the CodePage sector on a HPFS volume"
SAY
SAY "Example:"
SAY "  CP C:"
EXIT
