/* REXX */

/*
 *  File     :  FTPGET.CMD
 *  Purpose  :  Fetch a file from a remote host
 *
 *
 *  Modification History:
 *
 *  --Date--- --By-- --Reason------------------------------------------------
 *  96 JUL 11  TAB   Created script
 *  96 AUG 17  TAB   Assorted repairs
 *
 */


/*  Banner:                                                         */

PARSE ARG args


/*  Banner:                                                         */

SAY "FTP Get  v1.1"
SAY "Copyright (C) Tom Brown 1996, 1997.  All rights reserved."
SAY


/*  Defaults:                                                       */

result      = 0
defUser     = "ftp"
defPass     = VALUE('USER',,'OS2ENVIRONMENT')||'@localhost'
defLDir     = "."
defRDir     = "/"
defLFile    = "*"
defRFile    = defLFile
defMode     = "Binary"


/*  Usage:                                                          */

IF ARG() == 0 THEN
    DO
        SAY "Usage:"
        SAY
        SAY "ftpGet [parameters]"
        SAY
        SAY "Parameter:     Default:"
        SAY "----------     ----------------------------------"
        SAY "site           None"
        SAY "userID        " defUser
        SAY "password      " defPass
        SAY "LDir          " defLDir
        SAY "RDir          " defRDir
        SAY "LFile         " defLFile
        SAY "RFile         " defRFile
        SAY "mode          " defMode
        SAY "delete         off"
    END
ELSE
    DO
        /*  Fetch parameters                                                */

        site = getParam('-site',args)
        IF DATATYPE(site) == 'NUM' THEN
            DO
                SAY 'ERROR: no site specified'
                EXIT 4
            END

        userID   = getParam('-userID',args)
        IF DATATYPE(userID)   == 'NUM' THEN userID   = defUser

        password = getParam('-password',args)
        IF DATATYPE(password) == 'NUM' THEN password = defPass

        RDir     = getParam('-RDir',args)
        IF DATATYPE(RDir)     == 'NUM' THEN RDir     = defRDir

        LDir     = getParam('-LDir',args)
        IF DATATYPE(LDir)     == 'NUM' THEN LDir     = defLDir

        RFile    = getParam('-RFile',args)
        IF DATATYPE(RFile)    == 'NUM' THEN RFile    = defRFile

        LFile    = getParam('-LFile',args)
        IF DATATYPE(LFile)    == 'NUM' THEN LFile    = RFile

        mode     = getParam('-mode',args)
        IF DATATYPE(mode)     == 'NUM' THEN mode     = defMode

        verbose  = getParam('-v',args)
        IF DATATYPE(verbose)  <> 'NUM' THEN verbose=1

        delete   = getParam('-delete',args)
        IF DATATYPE(delete)   <> 'NUM' THEN delete=1


        /*  Fetch parameters                                                */

        IF verbose THEN
            DO
                SAY 'site       = ' site
                SAY 'userID     = ' userID
                SAY 'password   = ' password
                SAY 'LDir       = ' LDir
                SAY 'RDir       = ' RDir
                SAY 'LFile      = ' LFile
                SAY 'RFile      = ' RFile
                SAY 'mode       = ' mode
                SAY
            END


        /*  Execute the fetch                                               */

        /*      Load the REXX FTP API .DLL                                  */

        rc = RxFuncAdd("FtpLoadFuncs", "rxFtp", "FtpLoadFuncs")
        rc = FtpLoadFuncs()


        /*      Configure user information                                  */

        rc = FtpSetUser(site, userID, password)
        IF rc == 0 THEN SAY 'FtpSetUser() returned with rc =' rc

        rc = FtpChDir(RDir)
        IF rc > 0 THEN SAY 'FtpChDir() returned with rc =' rc

        rc = FtpLs(RFile, dir.)
        IF rc > 0 THEN SAY 'FtpLs() returned with rc =' rc


        /*      Can't find remote files                                     */

        IF dir.0 == 0 THEN
            DO
                SAY 'ERROR: file not found'
                RETURN 2
            END


        /*      Put into standard URL format                                */

        IF SUBSTR(RDir,1,1) <> '/' & SUBSTR(RDir,1,1) <> '\' THEN site = site||'/'

        /*      Fetch each file                                             */

        x=TIME('E')
        DO j = 1 TO dir.0


            SAY 'INFO: fetching -> ftp://'||site||RDir||'/'||dir.j

            IF dir.j = RFile THEN rc = FtpGet(LDir||'\'||LFile,dir.j,mode)
            ELSE                  rc = FtpGet(LDir||'\'||dir.j,dir.j,mode)

            IF rc > 0 THEN SAY 'FtpGet() returned with rc =' rc
            ELSE IF delete THEN
                DO
                    SAY 'INFO: deleting ->' dir.j
                    rc = FtpDelete(dir.j)
                END

        END
        time = TIME('E')


        /*      Close out all sessions                                      */

        rc = FtpLogoff()
        IF rc > 0 THEN SAY 'FtpLogoff() returned with rc =' rc


        /*      Output the elapsed time                                     */

        IF dir.0 > 1 THEN pl = 's'
        ELSE pl = ''

        IF time > 120 THEN
            DO
                units = 'minutes'
                time  = TRUNC(time / 60, 1)
            END
        ELSE
            DO
                units = 'seconds'
                time  = TRUNC(time, 2)
            END

        SAY 'INFO:' time units 'required to obtain file'||pl


        /*      Unload the REXX FTP API .DLL                                */

        rc = FtpDropFuncs()

    END

RETURN result


getParam: PROCEDURE

    PARSE ARG param,line

    param = TRANSLATE(param)
    uline = TRANSLATE(line)

    /* */

    index = POS(param,uline)
    IF index == 0 THEN result = 0
    ELSE
        DO
            DO j = 1 TO WORDS(uline)
                w = WORD(uline,j)
                IF SUBSTR(w,1,LENGTH(param)) == param THEN DO

                    index = POS('=',w)
                    IF index == 0 THEN index = POS(':',w)
                    IF index == 0 THEN result = 1
                    ELSE result = SUBSTR(WORD(line,j),index + 1)

                END
            END
        END


    RETURN result
