/* Shows bitmap locations and free space runs */
ARG drive .  /* First parm should always be drive */

IF drive = '' THEN CALL HELP
parmList = "? /? /H HELP A: B:"
IF WordPos(drive, parmList) \= 0 THEN CALL Help

/* Register external DLL functions */
CALL RxFuncAdd 'ReadSect','Sector','ReadSect'
CALL RxFuncAdd 'RxDate','RexxDate','RxDate'

/* Initialise Lookup Table*/
DO exponent = 0 TO 7
   bitValue.exponent = D2C(2**exponent)
END exponent

secString = ReadSect(drive, 16)   /*Read Superblk sec*/
freespaceBmpList = C2D(Reverse(Substr(secString,25,4)))
totalsecs = C2D(Reverse(Substr(secString,17,4)))

'@cls'
SAY
SAY "Inspecting drive" drive
SAY
/* LSN 25 = list of bitmap blocks */
CALL ShowDword " List of Bitmap secs",25

startOfListBlk = 0
    startOfBlk = 0
    bmpListBlk = ""
        bmpBlk = ""
getFacingBands = 0
     runNumber = 0
    byteOffset = 0
     runNumber = 0
/* Read in 4 secs of the list of sec-usage bmp blks */
DO secWithinBlk = freespaceBmpList TO freespaceBmpList+3
   temp = StartOfListBlk + secWithinBlk
   bmpListBlk = bmpListBlk||ReadSect(drive, temp)
END secWithinBlk

SAY
SAY "Space-Usage Bitmap Blocks:"
SAY "                                         Freespace Runs:"

/* Use dword pointers to bmps to read in 2KB bmp blks */
DO listOffset = 1 TO 2048 BY 4
   startDecStr = C2D(Reverse(Substr(bmpListBlk,ListOffset,4)))
   IF startDecStr = 0 THEN    /* No more bmps listed */
      DO
      IF getFacingBands = 1 THEN
         DO    /* Last data band had no facing data band */
         bmpSize = 2048
         CALL DetermineFreeruns
         LEAVE
         END

      LEAVE
      END
       
   /*Display a blank line when a new facing band occurs*/
   IF (ListOffset+7//8) = 0 THEN
      SAY

   CALL ShowBmpBlk listOffset

   DO secWithinBlk = 0 TO 3
      temp = StartOfBlk + secWithinBlk
      bmpBlk = bmpBlk||ReadSect(drive, temp)
   END secWithinBlk

   getFacingBands = getFacingBands + 1
   IF getFacingBands = 2 THEN /* Wait until you get both */
      DO                      /* bmps for the facing data*/
      bmpSize = 4096          /* bands since maximum extent*/
      CALL DetermineFreeruns  /* length is 16,380 KB */
      byteOffset = byteOffset+4096
      getFacingBands = 0
      bmpBlk = ""
      END

END listOffset

EXIT  /**************EXECUTION ENDS HERE**************/


FourBytes2Hex:  /* Given offset, return dword */
ARG  startPos
rearranged = Reverse(Substr(secString,startPos,4))
RETURN C2X(rearranged)


ShowDword:     /* Display dword and dec equivalent */
PARSE ARG label, offset
hexStr = FourBytes2Hex(offset)
SAY label": 0x"hexStr "("X2D(hexStr)")"
RETURN


ShowBmpBlk:
/* Show start-end of freespace runs in hex & dec */
PARSE ARG  offset
endDecStr = C2D(Reverse(Substr(bmpListBlk,offset,4)))+3
SAY "   0x"D2X(startDecStr,8)"-"D2X(endDecStr,8) " ("startDecStr"-"endDecStr")"
startOfBlk = startDecStr
RETURN


DetermineFreeruns:
runStatus = 0
oldchar = ''
/* Check 128 secs at a time to speed up operation */
DO para = 1 to bmpSize BY 16
   /* 16 bytes*8 secs/byte = 128 secs per para scanned */
   char = Substr(bmpBlk,para,16)
   IF char = 'FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF'x & runstatus = 1
      THEN ITERATE para

   IF char = '00000000000000000000000000000000'x & runstatus = 0
      THEN ITERATE para
      /* Part of paragraph has run start/end
         so check a byte (8 secs) at a time. */

   DO byte = para TO para + 15
      char = Substr(bmpBlk,byte,1)
      IF char > '0'x THEN        /* 1 or more free secs */
         DO
         IF char = 'FF'x THEN       /* 8 unoccupied secs */
            IF runStatus = 1 THEN  /* Run is in progress */
               NOP
            ELSE         /* Run starts on 8 sec boundary */
               DO
               startByte = byte + byteOffset
               startBitPos = 0
               runStatus = 1   /* Start run determination */
               END
         ELSE
            CALL DetermineBit /* Partial usage of 8 secs */
         END
      ELSE
         DO                    /* All 8 secs are used */
         IF runStatus = 1 THEN
            DO
            endByte = byte + byteOffset
            endBitPos = -1   /* Run ends with prior sec */
            CALL ShowRun
            END
         END
   END byte
END para

IF runStatus = 1 THEN   /* Freespace at end of part. */
   DO
   endByte = 9999999999   /* Larger than # of secs in   */
   endBitPos = 0          /* max. possible part.(512GB) */
   CALL ShowRun           /* so ShowRun will set runEnd */
   END                    /* to last LSN in this part.  */

RETURN


DetermineBit:  /* Free/occupied usage within 8 sec blk */
DO bitPos = 0 TO 7
   IF runStatus = 0 THEN
      DO                /* No run currently in progress */
      IF BitAnd(char, bitValue.bitPos) > '0'x THEN
         DO              /* sec is free */
         startByte = byte + byteOffset
         startBitPos = bitPos
         runStatus = 1
         END
      END
   ELSE
      DO
      IF BitAnd(char, bitValue.bitPos) = '0'x THEN
         DO              /* sec is used */
         endByte = byte + byteOffset
         /* When a run ends, the sec before the first
            used one is the last sec in the freerun. */
         endBitPos = bitPos - 1
         CALL ShowRun
         END
      END
END bitPos
RETURN


ShowRun:
/* Display freerun start-end secs & reset run status */
runNumber = runNumber + 1
runStart = (startByte - 1) * 8 + startBitPos
runEnd = (endByte - 1) * 8 + endBitPos

IF runEnd > totalSecs THEN runEnd = TotalSecs - 1
IF runStart \= runEnd THEN  /* More than 1 sec is free */
   DO
   run = runStart"-"runEnd
   run = Left(run||Copies(" ",14),15)
   SAY Copies(" ",40) run "(#"runNumber":"runEnd-RunStart+1")"
   END
ELSE
   DO
   run = Left(runStart||Copies(" ",14),15)
   SAY Copies(" ",40) run "(#"runNumber":1)"
   END

runStatus = 0
RETURN

Help:
SAY
SAY "Purpose:"
SAY "  ShowFreeruns displays the location of the sec-usage bitmap blocks"
SAY "   and the location and extent of free space runs."
SAY
SAY "Example:"
SAY "  ShowFreeruns C:"
SAY
EXIT
